/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.pdfextractor;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.pdfbox.contentstream.operator.Operator;
import org.apache.pdfbox.cos.COSArray;
import org.apache.pdfbox.cos.COSFloat;
import org.apache.pdfbox.cos.COSString;
import org.apache.pdfbox.pdfparser.PDFStreamParser;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.common.PDRectangle;

import de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPage;
import de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPageFirst;
import de.willuhn.logging.Logger;

/**
 * TextAreas fr die erste Seite.
 *
 * @author Frank Mnster
 */
public class SantanderPdfPageFirst extends AbstractBankPDFPageFirst {

    /** Erwartete X-Position des Buchungsdaten. */
    private static final double BUCH_DATUM_X_POS = 33.35;

    /** page footer height */
    private static final double PAGE_FOOTER_HEIGHT = 110.0;

    /** space between two lines */
    private static final double LINE_OFFSET = 20.0;

    /**
     * y-position of first possible transaction
     */
    private double yPosition;

    /** y height of ??? */
    private double yHeight;

    /**
     * Standard constructor call super class.
     *
     * @param page
     *            the first page of the account sheet to search the start of
     *            transactions
     *
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    public SantanderPdfPageFirst(final PDPage page) throws IOException {
        super();

        // re-caculate y-position of transactions
        searchTransactionPosition(page);

        // re-create the regions map
        final Map<String, Rectangle> regions = defineRegions(); // NOPMD on
                                                                // 22.02.21,
                                                                // 20:02

        // reset the regions map
        this.setRegion(AbstractBankPDFPage.BUCHUNGS_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.KAUF_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.BESCHREIBUNG, regions);
        this.setRegion(AbstractBankPDFPage.FREMDW_BETRAG, regions);
        this.setRegion(AbstractBankPDFPage.FREMDWAEHRUNG, regions);
        this.setRegion(AbstractBankPDFPage.KURS, regions);
        this.setRegion(AbstractBankPDFPage.BETRAG, regions);

    }

    /**
     * Define first page regions.
     *
     * @return the map
     * @see AbstractBankPDFPage#defineRegions
     */
    @Override
    protected Map<String, Rectangle> defineFirstPageRegions() {

        final Map<String, Rectangle> regions = new HashMap<>(); // NOPMD
        regions.put(AbstractBankPDFPage.KARTEN_KONTO, AbstractBankPDFPage.getRectangleFrom(12.5, 3.3, 4.0, 0.6));
        regions.put(AbstractBankPDFPage.KARTEN_INHABER, AbstractBankPDFPage.getRectangleFrom(12.5, 4.3, 4.0, 0.6));
        regions.put(AbstractBankPDFPage.RECHNUNGS_DATUM, AbstractBankPDFPage.getRectangleFrom(17.0, 6.5, 2.0, 0.5));
        regions.put(AbstractBankPDFPage.SALDO, AbstractBankPDFPage.getRectangleFrom(17.0, 7.6, 3.0, 0.8));
        regions.put(AbstractBankPDFPage.FAELLIG_BETRAG, AbstractBankPDFPage.getRectangleFrom(17.0, 8.3, 3.0, 0.8));
        regions.put(AbstractBankPDFPage.FAELLIG_DATUM, AbstractBankPDFPage.getRectangleFrom(17.0, 8.9, 3.0, 0.8));
        regions.put(AbstractBankPDFPage.FAELLIG_IBAN, AbstractBankPDFPage.getRectangleFrom(14.5, 27.8, 8.0, 0.3));
        regions.put(AbstractBankPDFPage.FAELLIG_BIC, AbstractBankPDFPage.getRectangleFrom(14.5, 28.1, 8.0, 0.3));
        return regions;
    }

    /**
     * Define regions.
     *
     * @return the map
     * @see AbstractBankPDFPage#defineRegions
     */
    @Override
    protected Map<String, Rectangle> defineRegions() {

        // calculate y-position in cm

        final double yPosInCm = this.yPosition / AbstractBankPDFPage.CM_TO_PDF;
        final double yHeightInCm = this.yHeight / AbstractBankPDFPage.CM_TO_PDF;

        final Map<String, Rectangle> regions = new HashMap<>(); // NOPMD
        regions.put(AbstractBankPDFPage.BUCHUNGS_DATUM,
                AbstractBankPDFPage.getRectangleFrom(1.1, yPosInCm, 1.2, yHeightInCm));
        regions.put(AbstractBankPDFPage.BESCHREIBUNG,
                AbstractBankPDFPage.getRectangleFrom(2.4, yPosInCm, 7.3, yHeightInCm));
        regions.put(AbstractBankPDFPage.KURS, AbstractBankPDFPage.getRectangleFrom(14.8, yPosInCm, 1.4, yHeightInCm));
        regions.put(AbstractBankPDFPage.FREMDWAEHRUNG,
                AbstractBankPDFPage.getRectangleFrom(10.1, yPosInCm, 1.9, yHeightInCm));
        regions.put(AbstractBankPDFPage.FREMDW_BETRAG,
                AbstractBankPDFPage.getRectangleFrom(10.1, yPosInCm, 1.9, yHeightInCm));
        regions.put(AbstractBankPDFPage.KAUF_DATUM,
                AbstractBankPDFPage.getRectangleFrom(16.2, yPosInCm, 1.4, yHeightInCm));
        regions.put(AbstractBankPDFPage.BETRAG, AbstractBankPDFPage.getRectangleFrom(17.8, yPosInCm, 2.1, yHeightInCm));
        return regions;
    }

    /**
     * Gets the x pos.
     *
     * @return the x pos
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPage#getXPos()
     */
    @Override
    protected double getXPos() {

        return SantanderPdfPageFirst.BUCH_DATUM_X_POS;
    }

    private String extractBlockText(final List<Object> block) {

        final StringBuilder text = new StringBuilder();

        for (final Object token : block) {

            if (token instanceof COSString) {
                final COSString tok = (COSString) token;
                text.append(tok.getString());
            } else if (token instanceof COSArray) {
                text.append(extractCOSArray((COSArray) token));
            }

        }

        return text.toString();

    }

    private StringBuilder extractCOSArray(final COSArray tokAry) {

        final StringBuilder text = new StringBuilder();

        for (final Object cosBase : tokAry) {
            if (cosBase instanceof COSString) {
                final COSString cosString = (COSString) cosBase;
                text.append(cosString.getString());
            }
        }

        return text;
    }

    /**
     * Read text between BT and ET token into a List<Objects>
     *
     * @param tokens
     * @return List<Objects>
     */
    private List<Object> readTokens(final PDFStreamParser parser) {

        // final List<Object> tokens

        boolean collect = false;
        final List<Object> block = new ArrayList<>();

        Object token = null;
        try {
            token = parser.parseNextToken();
        } catch (final IOException e) {
            Logger.warn("Parse error for next token :" + e.getLocalizedMessage()); //$NON-NLS-1$
        }

        while (token != null) {

            if (token instanceof Operator) {

                final Operator operator = (Operator) token;
                if (operator.getName().equals("BT")) { //$NON-NLS-1$
                    collect = true;
                    block.clear();

                } else if (operator.getName().equals("ET")) { //$NON-NLS-1$
                    collect = false;

                    final String blockText = extractBlockText(block);
                    Logger.debug("Block Text:" + blockText); //$NON-NLS-1$

                    // Break the loop as the datum text is found
                    if ("Buchungs-datum".equals(blockText)) { //$NON-NLS-1$
                        break;
                    }

                } else {
                    // add the operator as it is not BT or ET
                    block.add(token);
                }
            }

            // add the token as it is not an operator
            if (collect) {
                block.add(token);
            }

            // next token
            try {
                token = parser.parseNextToken();
            } catch (final IOException e) {
                Logger.warn("Parse error for net token :" + e.getLocalizedMessage()); //$NON-NLS-1$
                token = null;
            }
        }

        // return the list
        return block;
    }

    private void searchTransactionPosition(final PDPage page) {

        try {
            final PDFStreamParser parser = new PDFStreamParser(page);
            // parser.parse();
            // final List<Object> tokens = parser. .getTokens();

            // read tokens into a list
            final List<Object> block = readTokens(parser);

            // Block found --> get position
            setBlockPosition(block, page);

        } catch (final IOException e) {
            Logger.error("Fehler beim Paser des PDF auf nach der Suche des Buchungsdatum-Bereiches");
        }

    }

    private void setBlockPosition(final List<Object> block, final PDPage page) {

        final double yPosi = ((COSFloat) block.get(10)).floatValue();
        final PDRectangle cropBox = page.getCropBox();
        this.yPosition = cropBox.getHeight() - (yPosi - SantanderPdfPageFirst.LINE_OFFSET);
        this.yHeight = yPosi - SantanderPdfPageFirst.PAGE_FOOTER_HEIGHT; //

        Logger.debug("y-Position: " + this.yPosition); //$NON-NLS-1$
        Logger.debug("y-Hhe: " + this.yHeight); //$NON-NLS-1$

    }

}
