/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.Settings;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.FileInput;
import de.willuhn.jameica.hbci.gui.input.KontoInput;

/**
 * Controller for the general settings tab
 *
 * @author Frank Mnster
 */
public class GeneralSettingsController extends AbstractControl implements SettingsController {

    /**
     * FileInput to set the Selenium driver
     */
    private FileInput seleniumDriver;

    /**
     * FileInput to set the Selenium FF binary
     */
    private FileInput seleniumFfBinary;

    /**
     * Auto-close selenium window
     */
    private CheckboxInput seleniumAutoclose;

    /**
     * Selenium used debug log level
     */
    private CheckboxInput seleniumUseDebugLogLevel;

    /**
     * Constructor with super call
     */
    public GeneralSettingsController() {

        //
        super(null);
    }

    /**
     * Returns a {@link CheckboxInput} to set the Firefox window auto close
     * option.
     *
     * @return {@link CheckboxInput} field
     */
    public CheckboxInput getAutoCloseInput() {
        if (this.seleniumAutoclose == null) {
            this.seleniumAutoclose = new CheckboxInput(Settings.isSeleniumAutoClose());

        }
        return this.seleniumAutoclose;
    }

    /**
     * @see SettingsController#getKonto()
     */
    @Override
    public KontoInput getKonto() {

        return null;
    }

    /**
     * Returns a {@link CheckboxInput} to set the Selenium debug level option.
     *
     * @return {@link CheckboxInput} field
     */
    public CheckboxInput getSeleliumUseDebugLogLevel() {
        if (this.seleniumUseDebugLogLevel == null) {
            this.seleniumUseDebugLogLevel = new CheckboxInput(Settings.isSeleniumUseDegugLog());

        }
        return this.seleniumUseDebugLogLevel;
    }

    /**
     * Returns a {@link FileInput} to select the FireFox driver
     *
     * @return FileInput seleniumDriver
     */
    public FileInput getSeleniumDriver() {

        if (this.seleniumDriver == null) {
            final String driver = Settings.getSeleniumDriver();
            this.seleniumDriver = new FileInput(driver, false);
        }
        return this.seleniumDriver;
    }

    /**
     * Returns a {@link FileInput} to select the FireFox binary
     *
     * @return FileInput seleniumFfBinary
     */
    public FileInput getSeleniumFfBinary() {

        if (this.seleniumFfBinary == null) {
            final String driver = Settings.getSeleniumFireFoxBinary();
            this.seleniumFfBinary = new FileInput(driver, false);
        }
        return this.seleniumFfBinary;
    }

    /**
     * @see SettingsController#speichern()
     */
    @Override
    public void speichern() {

        // path to driver
        String path = (String) this.getSeleniumDriver().getValue();
        if (path == null) {
            path = StringUtils.EMPTY;
        }
        Settings.setSeleniumDriver(path);

        // path to binary
        path = (String) this.getSeleniumFfBinary().getValue();
        if (path == null) {
            path = StringUtils.EMPTY;
        }
        Settings.setSeleniumFireFoxBinary(path);

        // auto close the FF window after each jop group
        Boolean autoClose = (Boolean) this.getAutoCloseInput().getValue();
        if (autoClose == null) {
            autoClose = false;
        }
        Settings.setSeleniumAutoClose(autoClose); // auto close the FF window
                                                  // after each jop group

        // Use debug log level
        Boolean useDebugLogLevel = (Boolean) this.getSeleliumUseDebugLogLevel().getValue();
        if (useDebugLogLevel == null) {
            useDebugLogLevel = false;
        }
        Settings.setSeleniumUseDebugLog(useDebugLogLevel);

    }

    /**
     * General settings do not support account statements :-)
     */
    @Override
    public boolean supportsStatements() {

        return false;
    }

}
