/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.mahoe.gui.controller.GeneralSettingsController;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Part to show general settings of Mahoe
 *
 * @author Frank Mnster
 */
public class GeneralSettingsPart implements Part {

    /**
     * Our controller
     */
    private final GeneralSettingsController control;

    /**
     * Creates a BosSetting Part With all setting parameters for the Bank of
     * Scottland
     *
     * @param control
     *            the BosSettingsController
     */
    public GeneralSettingsPart(final GeneralSettingsController control) {

        this.control = control;
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        final ColumnLayout clContent = new ColumnLayout(parent, 2);
        final SimpleContainer left = new SimpleContainer(clContent.getComposite(), true);
        left.addText(AbstractSettingsPart.FIX_SPACE_TEXT, true);
        left.addHeadline(I18N.tr("GeneralSettingsPart.generalSettings")); //$NON-NLS-1$
        left.addText(AbstractSettingsPart.FIX_SPACE_TEXT, true);
        left.addLabelPair(I18N.tr("GeneralSettingsPart.seleniumFfDriver"), this.control.getSeleniumDriver()); //$NON-NLS-1$
        left.addText(I18N.tr("GeneralSettingsPart.snapFF"), true); //$NON-NLS-1$
        left.addLabelPair(I18N.tr("GeneralSettingsPart.seleniumFfDBinary"), this.control.getSeleniumFfBinary()); //$NON-NLS-1$
        // left.addLabelPair("Firefox Profile",
        // this.control.getFfProfileInput()); //$NON-NLS-1$
        left.addLabelPair(I18N.tr("GeneralSettingsPart.seleniumLogLevel"), this.control.getSeleliumUseDebugLogLevel()); //$NON-NLS-1$
        left.addLabelPair(I18N.tr("GeneralSettingsPart.seleniumFfAutoClose"), this.control.getAutoCloseInput()); //$NON-NLS-1$

    }

}
