/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.utils;

import java.net.URI;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;

/**
 * Utility class for web methods
 *
 * @author Frank Mnster
 */
public final class WebUtils {

    /**
     * HTML tag href
     */
    public static final String HTML_TAG_HREF = "href"; //$NON-NLS-1$

    /**
     * HTML tag "td"
     */
    public static final String HTML_TAG_TD = "td"; //$NON-NLS-1$

    /**
     * HTML tag for the first cell "td[1]"
     */
    public static final String HTML_TAG_TD1 = WebUtils.HTML_TAG_TD + "[1]"; //$NON-NLS-1$

    /**
     * HTML tag "tr"
     */
    public static final String HTML_TAG_TR = "tr"; //$NON-NLS-1$

    /**
     * HTML tag "table"
     */
    public static final String HTML_TAG_TABLE = "table"; //$NON-NLS-1$

    /**
     * HTML tag for table body "tbody"
     */
    public static final String HTML_TAG_TBODY = "tbody"; //$NON-NLS-1$

    /**
     * HTML tag anchor "a"
     */
    public static final String HTML_TAG_ANCHOR = "a"; //$NON-NLS-1$

    /**
     * HTML tag paragraph "p"
     */
    public static final String HTML_TAG_PARAGRAPH = "p"; //$NON-NLS-1$

    /**
     * HTML tag paragraph "li"
     */

    public static final String HTML_TAG_LINK = "li"; //$NON-NLS-1$

    /**
     * Returns the query parameters of an URI
     *
     * @param query
     * @return a map with all param value pairs
     */
    public static Map<String, String> getQueryMap(final URI uri) {

        final Map<String, String> map = new ConcurrentHashMap<>();
        final String query = uri.getQuery();
        if (query != null) {
            final String[] params = query.split("&"); //$NON-NLS-1$
            for (final String param : params) {
                final String name = param.split("=")[0]; //$NON-NLS-1$
                final String value = param.split("=")[1]; //$NON-NLS-1$
                map.put(name, value);
            }
        }
        return map;
    }

    /**
     * Check if String looks like JSON
     *
     * @param content
     *            the string to test
     * @return true in case content starts with {
     * @throws WebSiteException
     *             in case string does not start with {
     */
    public static Boolean isJson(final String content) throws WebSiteException {

        if (!content.isEmpty() && content.charAt(0) != '{') {
            Logger.error("Return String is no JSON"); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("BshWebSiteUtils.accountRequestFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
        }
        return true;
    }

    /**
     * private class constructor
     */
    private WebUtils() {

        // intentionally left blank
    }

}
